@@ -17,7 +17,7 @@ import typing
 from . import utils
 
 
-__version__ = '5.0.0'
+__version__ = '5.0.2'
 _log = logging.getLogger('calibr8')
 
 
@@ -52,12 +52,22 @@ class NumericPosterior(typing.NamedTuple):
             result += f"\n    .{name}:\t{type(value).__name__}"
         return result
 
+    @property
+    def hdi_lower(self) -> float:
+        """ Lower bound of the HDI. This is the first value in `x_dense`. """
+        return self.x_dense[0]
 
-class ErrorModel:
-    """A parent class providing the general structure of an error model."""
+    @property
+    def hdi_upper(self) -> float:
+        """ Upper bound of the HDI. This is the last value in `x_dense`. """
+        return self.x_dense[-1]
+
+
+class CalibrationModel:
+    """A parent class providing the general structure of a calibration model."""
     
     def __init__(self, independent_key:str, dependent_key:str, *, theta_names:typing.Tuple[str]):
-        """Creates an ErrorModel object.
+        """Creates a CalibrationModel object.
 
         Parameters
         ----------
@@ -232,7 +242,7 @@ class ErrorModel:
         return objective
 
     def save(self, filepath: os.PathLike):
-        """Save key properties of the error model to a JSON file.
+        """Save key properties of the calibration model to a JSON file.
 
         Parameters
         ----------
@@ -274,8 +284,8 @@ class ErrorModel:
 
         Returns
         -------
-        errormodel : ErrorModel
-            the instantiated error model
+        calibrationmodel : CalibrationModel
+            the instantiated calibration model
         """
         with open(filepath, 'r') as jfile:
             data = json.load(jfile)
