@@ -1,5 +1,6 @@
 import collections
 import datetime
+import logging
 import numpy
 import pathlib
 import pytest
@@ -7,6 +8,7 @@ import scipy
 import scipy.stats as stats
 
 import calibr8
+import calibr8.utils
 
 
 try:
@@ -27,7 +29,7 @@ except ModuleNotFoundError:
 dir_testfiles = pathlib.Path(pathlib.Path(__file__).absolute().parent, 'testfiles')
 
 
-class _TestModel(calibr8.ErrorModel):
+class _TestModel(calibr8.CalibrationModel):
     def __init__(self, independent_key=None, dependent_key=None, theta_names=None):
         if theta_names is None:
             theta_names = tuple('a,b,c'.split(','))
@@ -49,7 +51,7 @@ class _TestLogIndependentLogisticModel(calibr8.BaseLogIndependentAsymmetricLogis
         super().__init__(independent_key='I', dependent_key='D', scale_degree=scale_degree)
 
 
-class TestBasicErrorModel:
+class TestBasicCalibrationModel:
     def test_init(self):
         em = _TestModel('I', 'D', theta_names=tuple('c,d,e'.split(',')))
         assert em.independent_key == 'I'
@@ -65,18 +67,18 @@ class TestBasicErrorModel:
         pass
     
     def test_constructor_signature_check(self):
-        class EM_OK(calibr8.ErrorModel):
+        class EM_OK(calibr8.CalibrationModel):
             def __init__(self, arg1=1, *, kwonly=2, kwonlydefault=4):
                 super().__init__('I', 'D', theta_names=tuple('abc'))
         EM_OK()
 
-        class EM_args(calibr8.ErrorModel):
+        class EM_args(calibr8.CalibrationModel):
             def __init__(self, arg1):
                 super().__init__('I', 'D', theta_names=tuple('abc'))
         with pytest.raises(TypeError):
             EM_args(arg1=3)
 
-        class EM_kwargs(calibr8.ErrorModel):
+        class EM_kwargs(calibr8.CalibrationModel):
             def __init__(self, *, kwonly, kwonlydefault=4):
                 super().__init__('I', 'D', theta_names=tuple('abc'))
         with pytest.raises(TypeError):
@@ -89,15 +91,15 @@ class TestBasicErrorModel:
         dependent = 'BS'
         x = numpy.array([1,2,3])
         y = numpy.array([4,5,6])
-        errormodel = _TestModel()
+        cmodel = _TestModel()
         with pytest.raises(NotImplementedError):
-            _ = errormodel.predict_dependent(x)
+            _ = cmodel.predict_dependent(x)
         with pytest.raises(NotImplementedError):
-            _ = errormodel.predict_independent(x)
+            _ = cmodel.predict_independent(x)
         with pytest.raises(NotImplementedError):
-            _ = errormodel.infer_independent(y, lower=0, upper=10, steps=10, hdi_prob=None)
+            _ = cmodel.infer_independent(y, lower=0, upper=10, steps=10, hdi_prob=None)
         with pytest.raises(NotImplementedError):
-            _ = errormodel.loglikelihood(y=y, x=x, theta=[1,2,3])
+            _ = cmodel.loglikelihood(y=y, x=x, theta=[1,2,3])
         pass
 
     def test_save_and_load_version_check(self):
@@ -129,7 +131,7 @@ class TestBasicErrorModel:
         calibr8.core.__version__ = '.'.join(map(str, vactual))
         
         # load with the wrong model
-        class DifferentEM(calibr8.ErrorModel):
+        class DifferentEM(calibr8.CalibrationModel):
             pass
 
         with pytest.raises(calibr8.CompatibilityException):
@@ -374,22 +376,22 @@ class TestSymbolicModelFunctions:
 
 class TestUtils:
     def test_datetime_parsing(self):
-        assert calibr8.parse_datetime(None) is None
+        assert calibr8.utils.parse_datetime(None) is None
         assert (
-            calibr8.parse_datetime('2018-12-01T09:27:30Z')
+            calibr8.utils.parse_datetime('2018-12-01T09:27:30Z')
             ==
             datetime.datetime(2018, 12, 1, 9, 27, 30, tzinfo=datetime.timezone.utc)
         )
         assert (
-            calibr8.parse_datetime('2018-12-01T09:27:30+0000')
+            calibr8.utils.parse_datetime('2018-12-01T09:27:30+0000')
             ==
             datetime.datetime(2018, 12, 1, 9, 27, 30, tzinfo=datetime.timezone.utc)
         )
 
     def test_datetime_formatting(self):
-        assert calibr8.format_datetime(None) is None
+        assert calibr8.utils.format_datetime(None) is None
         assert (
-            calibr8.format_datetime(datetime.datetime(2018, 12, 1, 9, 27, 30, tzinfo=datetime.timezone.utc))
+            calibr8.utils.format_datetime(datetime.datetime(2018, 12, 1, 9, 27, 30, tzinfo=datetime.timezone.utc))
             ==
             '2018-12-01T09:27:30Z'
         )
@@ -423,12 +425,11 @@ class TestUtils:
         test_dict2 = {
             'a': 1, 
             'b': [1,2,3], 
-            'c': numpy.array([(1, tt.TensorVariable([1,2,3])), (3,4)])
+            'c': numpy.array([(1, tt.as_tensor_variable([1,2,3])), (3,4)])
         }
         assert (calibr8.istensor(test_dict2))
         assert (calibr8.istensor([1, tt.as_tensor_variable([1,2]), 3]))
-        assert (calibr8.istensor([1, tt.TensorVariable([1,2]), 3]))
-        assert (calibr8.istensor(numpy.array([1, tt.TensorVariable([1,2]), 3])))
+        assert (calibr8.istensor(numpy.array([1, tt.as_tensor_variable([1,2]), 3])))
 
     def test_import_warner(self):
         dummy = calibr8.utils.ImportWarner('dummy')
@@ -675,6 +676,8 @@ class TestBasePolynomialModelT:
         assert len(posterior.x_dense) == len(posterior.pdf)
         assert (posterior.hdi_prob==0.95)
         assert (numpy.isclose(scipy.integrate.cumtrapz(posterior.pdf,posterior.x_dense)[-1], 0.95, atol=0.0001))
+        assert posterior.hdi_lower == posterior.x_dense[0]
+        assert posterior.hdi_upper == posterior.x_dense[-1]
 
         # check that error are raised by wrong input
         with pytest.raises(ValueError):
@@ -685,17 +688,17 @@ class TestBasePolynomialModelT:
     
     @pytest.mark.skipif(not HAS_PYMC3, reason='requires PyMC3')
     def test_symbolic_loglikelihood(self):
-        errormodel = _TestPolynomialModel(independent_key='S', dependent_key='A', mu_degree=1, scale_degree=1)
-        errormodel.theta_fitted = [0, 1, 0.1, 1]
+        cmodel = _TestPolynomialModel(independent_key='S', dependent_key='A', mu_degree=1, scale_degree=1)
+        cmodel.theta_fitted = [0, 1, 0.1, 1]
        
         # create test data
         x_true = numpy.array([1,2,3,4,5])
-        y_obs = errormodel.predict_dependent(x_true)[0]
+        y_obs = cmodel.predict_dependent(x_true)[0]
 
-        # create a pymc3 model using the error model
+        # create a pymc3 model using the calibration model
         with pymc3.Model() as pmodel:
             x_hat = pymc3.Uniform('x_hat', lower=0, upper=10, shape=x_true.shape, transform=None)
-            L = errormodel.loglikelihood(x=x_hat, y=y_obs, replicate_id='A01', dependent_key='A')
+            L = cmodel.loglikelihood(x=x_hat, y=y_obs, replicate_id='A01', dependent_key='A')
             assert isinstance(L, tt.TensorVariable)
         
         # compare the two loglikelihood computation methods
@@ -703,32 +706,32 @@ class TestBasePolynomialModelT:
         actual = L.logp({
             'x_hat': x_test
         })
-        expected = errormodel.loglikelihood(x=x_test, y=y_obs)
+        expected = cmodel.loglikelihood(x=x_test, y=y_obs)
         numpy.testing.assert_almost_equal(actual, expected, 6)
         pass
 
     def test_loglikelihood(self):
         x = numpy.array([1,2,3])
         y = numpy.array([1,2,3])
-        errormodel = _TestPolynomialModel(independent_key='S', dependent_key='OD', mu_degree=1, scale_degree=1)
-        errormodel.theta_fitted = [0, 1, 0.1, 1.6]
+        cmodel = _TestPolynomialModel(independent_key='S', dependent_key='OD', mu_degree=1, scale_degree=1)
+        cmodel.theta_fitted = [0, 1, 0.1, 1.6]
         with pytest.raises(TypeError):
-            _ = errormodel.loglikelihood(y, x=x)
-        true = errormodel.loglikelihood(y=y, x=x)
-        mu, scale, df = errormodel.predict_dependent(x, theta=errormodel.theta_fitted)
+            _ = cmodel.loglikelihood(y, x=x)
+        true = cmodel.loglikelihood(y=y, x=x)
+        mu, scale, df = cmodel.predict_dependent(x, theta=cmodel.theta_fitted)
         expected = numpy.sum(stats.t.logpdf(x=y, loc=mu, scale=scale, df=df))
         assert expected == true
         x = 'hello'
         with pytest.raises(Exception):
-            _= errormodel.loglikelihood(y=y, x=x)
+            _= cmodel.loglikelihood(y=y, x=x)
         return
     
     def test_loglikelihood_without_fit(self):
         x = numpy.array([1,2,3])
         y = numpy.array([1,2,3])
-        errormodel = _TestPolynomialModel(independent_key='Glu', dependent_key='OD', mu_degree=1, scale_degree=1)
+        cmodel = _TestPolynomialModel(independent_key='Glu', dependent_key='OD', mu_degree=1, scale_degree=1)
         with pytest.raises(Exception):
-            _= errormodel.loglikelihood(y=y, x=x)
+            _= cmodel.loglikelihood(y=y, x=x)
         return
 
 
@@ -736,38 +739,38 @@ class TestBaseAsymmetricLogisticModelT:
     def test_predict_dependent(self):
         x = numpy.array([1,2,3])
         theta = [0, 4, 2, 1, 1, 0, 2, 1.4]
-        errormodel = _TestLogisticModel(independent_key='S', dependent_key='OD', scale_degree=1)
-        errormodel.theta_fitted = theta
+        cmodel = _TestLogisticModel(independent_key='S', dependent_key='OD', scale_degree=1)
+        cmodel.theta_fitted = theta
         with pytest.raises(TypeError):
-            _ = errormodel.predict_dependent(x, theta)
-        mu, scale, df = errormodel.predict_dependent(x)
+            _ = cmodel.predict_dependent(x, theta)
+        mu, scale, df = cmodel.predict_dependent(x)
         numpy.testing.assert_array_equal(mu, calibr8.asymmetric_logistic(x, theta))
         numpy.testing.assert_array_equal(scale, 0 + 2 * mu)
         assert df == 1.4
         return
     
     def test_predict_independent(self):
-        errormodel = _TestLogisticModel(independent_key='S', dependent_key='OD', scale_degree=1)
-        errormodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.43]
+        cmodel = _TestLogisticModel(independent_key='S', dependent_key='OD', scale_degree=1)
+        cmodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.43]
         x_original = numpy.array([4, 5, 6])
-        mu, sd, df = errormodel.predict_dependent(x_original)
-        x_predicted = errormodel.predict_independent(y=mu)
+        mu, sd, df = cmodel.predict_dependent(x_original)
+        x_predicted = cmodel.predict_independent(y=mu)
         assert (numpy.allclose(x_predicted, x_original))
         return
     
     @pytest.mark.skipif(not HAS_PYMC3, reason='requires PyMC3')
     def test_symbolic_loglikelihood(self):
-        errormodel = _TestLogisticModel(independent_key='S', dependent_key='A', scale_degree=1)
-        errormodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.23]
+        cmodel = _TestLogisticModel(independent_key='S', dependent_key='A', scale_degree=1)
+        cmodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.23]
        
         # create test data
         x_true = numpy.array([1,2,3,4,5])
-        y_obs = errormodel.predict_dependent(x_true)[0]
+        y_obs = cmodel.predict_dependent(x_true)[0]
 
-        # create a pymc3 model using the error model
+        # create a pymc3 model using the calibration model
         with pymc3.Model() as pmodel:
             x_hat = pymc3.Uniform('x_hat', lower=0, upper=10, shape=x_true.shape, transform=None)
-            L = errormodel.loglikelihood(x=x_hat, y=y_obs, replicate_id='A01', dependent_key='A')
+            L = cmodel.loglikelihood(x=x_hat, y=y_obs, replicate_id='A01', dependent_key='A')
             assert isinstance(L, tt.TensorVariable)
         
         # compare the two loglikelihood computation methods
@@ -775,32 +778,32 @@ class TestBaseAsymmetricLogisticModelT:
         actual = L.logp({
             'x_hat': x_test
         })
-        expected = errormodel.loglikelihood(x=x_test, y=y_obs)
+        expected = cmodel.loglikelihood(x=x_test, y=y_obs)
         numpy.testing.assert_almost_equal(actual, expected, 6)
         return
     
     def test_loglikelihood(self):
         x = numpy.array([1,2,3])
         y = numpy.array([1,2,3])
-        errormodel = _TestLogisticModel(independent_key='S', dependent_key='OD', scale_degree=1)
-        errormodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.7]
+        cmodel = _TestLogisticModel(independent_key='S', dependent_key='OD', scale_degree=1)
+        cmodel.theta_fitted = [0, 4, 2, 1, 1, 2, 1.7]
         with pytest.raises(TypeError):
-            _ = errormodel.loglikelihood(y, x=x)
-        true = errormodel.loglikelihood(y=y, x=x)
-        mu, scale, df = errormodel.predict_dependent(x, theta=errormodel.theta_fitted)
+            _ = cmodel.loglikelihood(y, x=x)
+        true = cmodel.loglikelihood(y=y, x=x)
+        mu, scale, df = cmodel.predict_dependent(x, theta=cmodel.theta_fitted)
         expected = numpy.sum(stats.t.logpdf(x=y, loc=mu, scale=scale, df=df))
         assert expected == true
         x = 'hello'
         with pytest.raises(Exception):
-            _= errormodel.loglikelihood(y=y, x=x)
+            _= cmodel.loglikelihood(y=y, x=x)
         return
     
     def test_loglikelihood_without_fit(self):
         x = numpy.array([1,2,3])
         y = numpy.array([1,2,3])
-        errormodel = _TestLogisticModel(independent_key='Glu', dependent_key='OD', scale_degree=1)
+        cmodel = _TestLogisticModel(independent_key='Glu', dependent_key='OD', scale_degree=1)
         with pytest.raises(Exception):
-            _= errormodel.loglikelihood(y=y, x=x)
+            _= cmodel.loglikelihood(y=y, x=x)
         return
 
 
@@ -820,6 +823,34 @@ class TestOptimization:
         em = _TestPolynomialModel()
         return theta_mu, theta_scale, theta, em, x, y
 
+    def test_finite_masking(self, caplog):
+        x = numpy.random.normal(size=5)
+        y = x ** 2
+        result = calibr8.optimization._mask_and_warn_inf_or_nan(x, y)
+        numpy.testing.assert_array_equal(result[0], x)
+        numpy.testing.assert_array_equal(result[1], y)
+
+        x[2] = float("nan")
+        with caplog.at_level(logging.WARNING):
+            result = calibr8.optimization._mask_and_warn_inf_or_nan(x, y, on="x")
+        numpy.testing.assert_array_equal(result[0], x[~numpy.isnan(x)])
+        numpy.testing.assert_array_equal(result[1], y[~numpy.isnan(x)])
+        assert "1 elements" in caplog.text
+
+        y[[0, 3]] = float("nan")
+        with caplog.at_level(logging.WARNING):
+            result = calibr8.optimization._mask_and_warn_inf_or_nan(x, y, on="y")
+        numpy.testing.assert_array_equal(result[0], x[~numpy.isnan(y)])
+        numpy.testing.assert_array_equal(result[1], y[~numpy.isnan(y)])
+        assert "2 elements" in caplog.text
+
+        with caplog.at_level(logging.WARNING):
+            result = calibr8.optimization._mask_and_warn_inf_or_nan(x, y)
+        assert not numpy.any(numpy.isnan(result[0]))
+        assert not numpy.any(numpy.isnan(result[1]))
+        assert "3 elements" in caplog.text
+        pass
+
     def test_fit_checks_guess_and_bounds_count(self):
         theta_mu, theta_scale, theta, em, x, y = self._get_test_model()
         common = dict(model=em, independent=x, dependent=y)
@@ -832,7 +863,7 @@ class TestOptimization:
                 fit(**common, theta_guess=numpy.ones_like(theta), theta_bounds=[(-5, 5)]*14)
         return
 
-    def test_fit_scipy(self):
+    def test_fit_scipy(self, caplog):
         numpy.random.seed(1234)
         theta_mu, theta_scale, theta, em, x, y = self._get_test_model()
         theta_fit, history = calibr8.fit_scipy(
@@ -849,16 +880,32 @@ class TestOptimization:
         assert em.theta_guess is not None
         numpy.testing.assert_array_equal(em.cal_independent, x)
         numpy.testing.assert_array_equal(em.cal_dependent, y)
+
+        with caplog.at_level(logging.WARNING):
+            x[0] = float("nan")
+            y[-1] = numpy.inf
+            calibr8.fit_scipy(
+                em,
+                independent=x, dependent=y,
+                theta_guess=numpy.ones_like(theta),
+                theta_bounds=[(-5, 5)]*len(theta_mu) + [(0.02, 1), (1, 20)]
+            )
+        assert "2 elements" in caplog.text
+        # inf/nan should only be ignored for fitting
+        numpy.testing.assert_array_equal(em.cal_independent, x)
+        numpy.testing.assert_array_equal(em.cal_dependent, y)
         pass
 
+    @pytest.mark.xfail(reason="PyGMO and PyMC3 have dependencies (cloudpickle/dill) that are currently incompatible. See https://github.com/uqfoundation/dill/issues/383")
     @pytest.mark.skipif(not HAS_PYGMO, reason='requires PyGMO')
-    def test_fit_pygmo(self):
+    def test_fit_pygmo(self, caplog):
         numpy.random.seed(1234)
         theta_mu, theta_scale, theta, em, x, y = self._get_test_model()
         theta_fit, history = calibr8.fit_pygmo(
             em,
             independent=x, dependent=y,
-            theta_bounds=[(-5, 5)]*len(theta_mu) + [(0.02, 1), (1, 20)]
+            theta_bounds=[(-5, 5)]*len(theta_mu) + [(0.02, 1), (1, 20)],
+            evolutions=5,
         )
         for actual, desired, atol in zip(theta_fit, theta, [0.10, 0.05, 0.2, 2]):
             numpy.testing.assert_allclose(actual, desired, atol=atol)
@@ -868,4 +915,18 @@ class TestOptimization:
         assert em.theta_guess is None
         numpy.testing.assert_array_equal(em.cal_independent, x)
         numpy.testing.assert_array_equal(em.cal_dependent, y)
+
+        with caplog.at_level(logging.WARNING):
+            x[0] = -numpy.inf
+            y[-1] = float("nan")
+            calibr8.fit_pygmo(
+                em,
+                independent=x, dependent=y,
+                theta_bounds=[(-5, 5)]*len(theta_mu) + [(0.02, 1), (1, 20)],
+                evolutions=5,
+            )
+        assert "2 elements" in caplog.text
+        # inf/nan should only be ignored for fitting
+        numpy.testing.assert_array_equal(em.cal_independent, x)
+        numpy.testing.assert_array_equal(em.cal_dependent, y)
         pass
